#ifndef spotflow_SDK_H
#define spotflow_SDK_H

/* This file is autogenerated. Don't modify it manually. */

#include <stdint.h>

/**
 * Device SDK for Spotflow IoT Platform
 * 
 * Version: 2.0.1
 * Documentation: https://docs.spotflow.io/device-sdk/c/
 */

/**
 * A client connected to the Platform. This object is managed by the Device SDK. Create its instance
 * using @ref spotflow_client_start and delete it using @ref spotflow_client_destroy.
 *
 * The client stores all outgoing communication to the local database file and then sends it in a
 * background thread asynchronously. Thanks to that, it works even when the connection is unreliable.
 * Similarly, the client also stores all ingoing communication to the local database file and deletes it
 * only after the application processes it.
 */
typedef struct spotflow_client_t spotflow_client_t;

/**
 * The maximum number of bytes of any [Device ID](https://docs.spotflow.io/connect-devices/#device-id) string
 * including the trailing NUL character.
 */
#define SPOTFLOW_DEVICE_ID_MAX_LENGTH 128

/**
 * The maximum number of bytes of any [Workspace ID](https://docs.spotflow.io/manage-access/workspaces/) string
 * including the trailing NUL character.
 */
#define SPOTFLOW_WORKSPACE_ID_MAX_LENGTH 37

/**
 * The maximum number of bytes of any error message including the trailing null character.
 */
#define SPOTFLOW_ERROR_MAX_LENGTH 1024

/**
 * A result of a function that can fail. If a function returns a @ref spotflow_result_t, the caller must
 * check that the returned value is @ref SPOTFLOW_OK before continuing further. If the function returns
 * a different value, the caller can call @ref spotflow_read_last_error_message to retrieve the error message.
 * Check the documentation of the particular function to see what results it can return.
 */
typedef enum spotflow_result_t {
    /**
     * The function succeeded.
     */
    SPOTFLOW_OK = 0,
    /**
     * The function failed with an error.
     */
    SPOTFLOW_ERROR,
    /**
     * A buffer provided to the function is too small.
     */
    SPOTFLOW_INSUFFICIENT_BUFFER,
    /**
     * The client cannot provide the response because it is not connected to the platform.
     */
    SPOTFLOW_NOT_READY,
} spotflow_result_t;

/**
 * The compression to use for sending a message.
 */
typedef enum spotflow_compression_t {
    /**
     * Do not compress the message.
     */
    SPOTFLOW_COMPRESSION_NONE = 0,
    /**
     * Compress the message using the fastest compression algorithm settings.
     */
    SPOTFLOW_COMPRESSION_FASTEST,
    /**
     * Compress the message using the algorithm settings that produce the smallest size.
     * Beware that this may be significantly slower than the fastest compression.
     * We recommend to test the performance of your application with this setting before using it in production.
     */
    SPOTFLOW_COMPRESSION_SMALLEST_SIZE,
} spotflow_compression_t;

/**
 * The verbosity levels of logging.
 */
typedef enum spotflow_log_level_t {
    /**
     * No logging.
     */
    SPOTFLOW_LOG_OFF = 0,
    /**
     * Log errors only.
     */
    SPOTFLOW_LOG_ERROR = 1,
    /**
     * Log errors and warnings.
     */
    SPOTFLOW_LOG_WARN = 2,
    /**
     * Log errors, warnings, and information.
     */
    SPOTFLOW_LOG_INFO = 3,
    /**
     * Log errors, warnings, information, and debug messages.
     */
    SPOTFLOW_LOG_DEBUG = 4,
    /**
     * Log errors, warnings, information, debug messages, and trace messages.
     */
    SPOTFLOW_LOG_TRACE = 5,
} spotflow_log_level_t;

/**
 * A set of options that specify how to connect to the Platform. This object is managed by the Device SDK.
 * Create its instance using @ref spotflow_client_options_create and delete it using @ref spotflow_client_options_destroy.
 * After you configure all the options, pass the address of @ref spotflow_client_options_t to
 * @ref spotflow_client_start.
 */
typedef struct spotflow_client_options_t spotflow_client_options_t;

/**
 * A set of options for sending [Messages](https://docs.spotflow.io/send-data/#message) to
 * a [Stream](https://docs.spotflow.io/send-data/#stream). This object is
 * managed by the Device SDK. Create its instance using @ref spotflow_message_context_create and delete it using
 * @ref spotflow_message_context_destroy.
 */
typedef struct spotflow_message_context_t spotflow_message_context_t;

/**
 * The summary of an ongoing [Provisioning Operation](https://docs.spotflow.io/connect-devices/#provisioning-operation).
 * This object is managed by the Device SDK and its contents must not be modified.
 *
 * If you specify a custom callback to @ref spotflow_client_options_set_display_provisioning_operation_callback,
 * you'll receive a pointer to @ref spotflow_provisioning_operation_t as its argument. The pointer is valid
 * only for the duration of the callback.
 */
typedef struct spotflow_provisioning_operation_t {
    /**
     * (Don't modify) The ID of this [Provisioning Operation](https://docs.spotflow.io/connect-devices/#provisioning-operation).
     */
    const char *id;
    /**
     * (Don't modify) The verification code of this [Provisioning Operation](https://docs.spotflow.io/connect-devices/#provisioning-operation).
     */
    const char *verification_code;
    /**
     * (Don't modify) The expiration time of this [Provisioning Operation](https://docs.spotflow.io/connect-devices/#provisioning-operation).
     * The operation is no longer valid after that.
     *
     * The date/time format is [RFC 3339](https://www.rfc-editor.org/rfc/rfc3339#section-5.8).
     */
    const char *expiration_time;
} spotflow_provisioning_operation_t;

/**
 * The callback to display the details of an ongoing [Provisioning Operation](https://docs.spotflow.io/connect-devices/#provisioning-operation)
 * when the Device SDK is performing [Device Provisioning](https://docs.spotflow.io/connect-devices/#device-provisioning).
 * The callback is called only if you have configured it by @ref spotflow_client_options_set_display_provisioning_operation_callback.
 * The callback is called on the same thread that calls @ref spotflow_client_start.
 *
 * @param operation The summary of the [Provisioning Operation](https://docs.spotflow.io/connect-devices/#provisioning-operation).
 *                  See @ref spotflow_provisioning_operation_t for details.
 * @param context The optional context that was configured by @ref spotflow_client_options_set_display_provisioning_operation_callback.
 */
typedef void (*spotflow_display_provisioning_operation_callback_t)(const struct spotflow_provisioning_operation_t *operation, void *context);

/**
 * Set the verbosity level of logging (@ref SPOTFLOW_LOG_WARN by default).
 *
 * @param level The verbosity level of logging.
 * @return @ref SPOTFLOW_OK if successful, @ref SPOTFLOW_ERROR otherwise.
 */
enum spotflow_result_t spotflow_set_log_level(enum spotflow_log_level_t level);

/**
 * Write the most recent error message into the provided buffer as a UTF-8 string, returning the
 * number of bytes written. If successful, the error message is consumed and will not be returned
 * again in a future call to this function.
 *
 * Since the string is in the UTF-8 encoding, Windows users may need to convert it to a UTF-16
 * string before displaying it.
 *
 * @param buffer The buffer to write the error message into.
 * @param buffer_length The length of the buffer in bytes. Use @ref SPOTFLOW_ERROR_MAX_LENGTH to be sure that it is
 *                      always large enough.
 * @return The number of bytes written into the buffer including the trailing null character.
 *         If no error has been produced yet, returns 0. If `buffer` or `buffer_length` are invalid
 *         (for example, null pointer or insufficient length), returns -1.
 */
int spotflow_read_last_error_message(char *buffer,
                                     size_t buffer_length);

/**
 * Create an object that stores the connection options. The created object of type @ref spotflow_client_options_t
 * is managed by the Device SDK. After you configure all the options (using either this function or with the additional
 * functions listed below), pass the address of @ref spotflow_client_options_t to @ref spotflow_client_start.
 * Delete @ref spotflow_client_options_t using @ref spotflow_client_options_destroy.
 *
 * @see spotflow_client_options_set_database_file
 *      spotflow_client_options_set_provisioning_token
 *      spotflow_client_options_set_device_id
 *      spotflow_client_options_set_instance
 *      spotflow_client_options_set_display_provisioning_operation_callback
 *
 * @param options (Output) The pointer to the @ref spotflow_client_options_t object that will be created by this function.
 * @param device_id (Optional) The [ID of the Device](https://docs.spotflow.io/connect-devices/#device-id) you
 *                  are running the code from. Use `NULL` if you don't want to specify it. See @ref spotflow_client_options_set_device_id.
 * @param provisioning_token The [Provisioning Token](https://docs.spotflow.io/connect-devices/#provisioning-token)
 *                           that will @ref spotflow_client_start use to start [Device Provisioning](https://docs.spotflow.io/connect-devices/#device-provisioning).
 *                           See @ref spotflow_client_options_set_provisioning_token.
 * @param database_file The path to the local database file where the Device SDK stores the connection credentials and
 *                     temporarily persists incoming and outgoing messages. See @ref spotflow_client_options_set_database_file.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid (or `NULL` if it's required).
 */
enum spotflow_result_t spotflow_client_options_create(struct spotflow_client_options_t **options,
                                                      const char *device_id,
                                                      const char *provisioning_token,
                                                      const char *database_file);

/**
 * Set the path to the local database file where the Device SDK stores the connection credentials and temporarily persists
 * incoming and outgoing messages. @ref spotflow_client_start creates the file if it doesn't exist.
 *
 * The file must end with the suffix `".db"`, for example, `"spotflow.db"`.
 * If you don't use an absolute path, the file is created relative to the current working directory.
 *
 * @param options The @ref spotflow_client_options_t object.
 * @param database_file The path to the local database file.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid.
 */
enum spotflow_result_t spotflow_client_options_set_database_file(struct spotflow_client_options_t *options,
                                                                 const char *database_file);

/**
 * Set the [Provisioning Token](https://docs.spotflow.io/connect-devices/#provisioning-token) that
 * will @ref spotflow_client_start use to start [Device Provisioning](https://docs.spotflow.io/connect-devices/#device-provisioning).
 *
 * See [Get Started](https://docs.spotflow.io/get-started) for instructions
 * [how to create a Provisioning Token](https://docs.spotflow.io/get-started#3-create-provisioning-token).
 *
 * @param options The @ref spotflow_client_options_t object.
 * @param provisioning_token The [Provisioning Token](https://docs.spotflow.io/connect-devices/#provisioning-token).
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid.
 */
enum spotflow_result_t spotflow_client_options_set_provisioning_token(struct spotflow_client_options_t *options,
                                                                      const char *provisioning_token);

/**
 * Set the [ID of the Device](https://docs.spotflow.io/connect-devices/#device-id) you
 * are running the code from. If you don't specify it here, you'll need to choose it during the approval of the
 * [Provisioning Operation](https://docs.spotflow.io/connect-devices/#provisioning-operation).
 * See [Device Provisioning](https://docs.spotflow.io/connect-devices/#device-provisioning) for
 * more details about the process.
 *
 * Make sure that no two [Devices](https://docs.spotflow.io/connect-devices/#device) in
 * the same [Workspace](https://docs.spotflow.io/manage-access/workspaces/) use the same ID.
 * Otherwise, unexpected errors can occur during the communication with the Platform.
 *
 * @param options The @ref spotflow_client_options_t object.
 * @param device_id (Optional) The [ID of the Device](https://docs.spotflow.io/connect-devices/#device-id).
 *                  Use `NULL` if you don't want to specify it.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid.
 */
enum spotflow_result_t spotflow_client_options_set_device_id(struct spotflow_client_options_t *options,
                                                             const char *device_id);

/**
 * Set the URI/hostname of the Platform instance where the
 * [Device](https://docs.spotflow.io/connect-devices/#device) will connect to.
 *
 * If your company uses a dedicated instance of the Platform, such as `acme.spotflow.io`, specify it here.
 * The default value is `api.eu1.spotflow.io`.
 *
 * @param options The @ref spotflow_client_options_t object.
 * @param instance (Optional) The domain of the Platform instance. Use `NULL` if you want to keep the default value.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid.
 */
enum spotflow_result_t spotflow_client_options_set_instance(struct spotflow_client_options_t *options,
                                                            const char *instance);

/**
 * Set the function that displays the details of the
 * [Provisioning Operation](https://docs.spotflow.io/connect-devices/#provisioning-operation) when
 * @ref spotflow_client_start is performing [Device Provisioning](https://docs.spotflow.io/connect-devices/#device-provisioning).
 * See [Get Started](https://docs.spotflow.io/get-started#3-create-provisioning-token) for hands-on experience with this process.
 *
 * @param options The @ref spotflow_client_options_t object.
 * @param callback (Optional) The function that displays the details of the [Provisioning Operation](https://docs.spotflow.io/connect-devices/#provisioning-operation).
 *                 Use `NULL` if you don't want to specify it.
 * @param context (Optional) The context that will be passed to `callback`. Use `NULL`
 *                if you don't want to specify it.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid.
 */
enum spotflow_result_t spotflow_client_options_set_display_provisioning_operation_callback(struct spotflow_client_options_t *options,
                                                                                           spotflow_display_provisioning_operation_callback_t callback,
                                                                                           void *context);

/**
 * Destroy the @ref spotflow_client_options_t object.
 *
 * @param options The @ref spotflow_client_options_t object to destroy.
 */
void spotflow_client_options_destroy(struct spotflow_client_options_t *options);

/**
 * Start communicating with the Platform. The created object of type @ref spotflow_client_t is managed by
 * the Device SDK. Delete it using @ref spotflow_client_destroy.
 *
 * If the [Device](https://docs.spotflow.io/connect-devices/#device) is
 * not yet registered in the Platform, or its
 * [Registration Token](https://docs.spotflow.io/connect-devices/#registration-token) is
 * expired, this method performs [Device Provisioning](https://docs.spotflow.io/connect-devices/#device-provisioning)
 * and waits for the approval.
 * See [Get Started](https://docs.spotflow.io/get-started) for the instructions how to
 * [set up the Platform](https://docs.spotflow.io/get-started#3-create-provisioning-token),
 * [start Device Provisioning](https://docs.spotflow.io/get-started#41-use-device-sdk),
 * and [approve the Device](https://docs.spotflow.io/get-started#42-approve-device).
 *
 * If the [Registration Token](https://docs.spotflow.io/connect-devices/#registration-token) from
 * the last run is still valid, this method succeeds even without the connection to the Internet. The Device SDK will
 * store all outgoing communication in the local database file and send it once it connects to the Platform.
 *
 * @param client (Output) The pointer to the @ref spotflow_client_t object that will be created by this function.
 * @param options The options that specify how to connect to the Platform.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid.
 */
enum spotflow_result_t spotflow_client_start(spotflow_client_t **client,
                                             const struct spotflow_client_options_t *options);

/**
 * Create an object that stores the options for sending [Messages](https://docs.spotflow.io/send-data/#message) to
 * the Platform.
 *
 * Initialize the @ref spotflow_message_context_t object: set the most important fields to the provided values and
 * the other fields to default values. The created object of type @ref spotflow_message_context_t is managed by the
 * Device SDK. You can configure the object either using this function or using the additional functions listed below.
 * Delete it using @ref spotflow_message_context_destroy.
 *
 * @see spotflow_message_context_set_stream_group
 *      spotflow_message_context_set_stream
 *      spotflow_message_context_set_compression
 *
 * @param message_context (Output) The pointer to the @ref spotflow_message_context_t object that will be created by this function.
 * @param stream_group (Optional) The [Stream Group](https://docs.spotflow.io/send-data/#stream-group)
 *                     the [Message](https://docs.spotflow.io/send-data/#message) will be sent to.
 *                     If `NULL`, the Platforms directs the Messages to the default Stream Group of the current
 *                     [Workspace](https://docs.spotflow.io/manage-access/workspaces/).
 * @param stream (Optional) The [Stream](https://docs.spotflow.io/send-data/#stream)
 *               the [Message](https://docs.spotflow.io/send-data/#message) will be sent to.
 *               If `NULL`, the Platform directs the Messages into the default Stream of the given Stream Group.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid.
 */
enum spotflow_result_t spotflow_message_context_create(struct spotflow_message_context_t **message_context,
                                                       const char *stream_group,
                                                       const char *stream);

/**
 * Set the [Stream Group](https://docs.spotflow.io/send-data/#stream-group) where
 * [Messages](https://docs.spotflow.io/send-data/#message) will be sent to.
 *
 * @param message_context The @ref spotflow_message_context_t object.
 * @param stream_group The [Stream Group](https://docs.spotflow.io/send-data/#stream-group).
 *                     If `NULL`, the Platforms directs the Messages to the default Stream Group of the current
 *                     [Workspace](https://docs.spotflow.io/manage-access/workspaces/).
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid.
 */
enum spotflow_result_t spotflow_message_context_set_stream_group(struct spotflow_message_context_t *message_context,
                                                                 const char *stream_group);

/**
 * Set the [Stream](https://docs.spotflow.io/send-data/#stream) where
 * [Messages](https://docs.spotflow.io/send-data/#message) will be sent to.
 *
 * @param message_context The @ref spotflow_message_context_t object.
 * @param stream The [Stream](https://docs.spotflow.io/send-data/#stream).
 *               If `NULL`, the Platform directs the Messages into the default Stream of the given Stream Group.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid.
 */
enum spotflow_result_t spotflow_message_context_set_stream(struct spotflow_message_context_t *message_context,
                                                           const char *stream);

/**
 * Set the compression to use for sending [Messages](https://docs.spotflow.io/send-data/#message).
 *
 * @param message_context The @ref spotflow_message_context_t object.
 * @param compression The compression to use for sending [Messages](https://docs.spotflow.io/send-data/#message).
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid.
 */
enum spotflow_result_t spotflow_message_context_set_compression(struct spotflow_message_context_t *message_context,
                                                                enum spotflow_compression_t compression);

/**
 * Destroy the @ref spotflow_message_context_t object.
 *
 * @param message_context The @ref spotflow_message_context_t object to destroy.
 */
void spotflow_message_context_destroy(struct spotflow_message_context_t *message_context);

/**
 * Enqueue a [Message](https://docs.spotflow.io/send-data/#message) to
 * be sent to the Platform.
 *
 * If the [Stream](https://docs.spotflow.io/send-data/#stream) doesn't have a
 * [Message ID Autofill Pattern](https://docs.spotflow.io/send-data/#message-id-autofill-pattern),
 * you must provide `message_id`.
 * If the [Stream](https://docs.spotflow.io/send-data/#stream) groups
 * [Messages](https://docs.spotflow.io/send-data/#message) into
 * [Batches](https://docs.spotflow.io/send-data/#batch) and doesn't have a
 * [Batch ID Autofill Pattern](https://docs.spotflow.io/send-data/#batch-id-autofill-pattern),
 * you must provide `batch_id`. See [User Guide](https://docs.spotflow.io/send-data/) for
 * more details.
 *
 * The method returns right after it saves the [Message](https://docs.spotflow.io/send-data/#message) to
 * the queue in the local database file. A background thread asynchronously sends the messages from the queue to the Platform.
 * You can check the number of pending messages in the queue using @ref spotflow_client_get_pending_messages_count.
 *
 * @param client The @ref spotflow_client_t object.
 * @param message_context The options that specify how to send the [Message](https://docs.spotflow.io/send-data/#message).
 * @param batch_id (Optional) The ID of the [Batch](https://docs.spotflow.io/send-data/#batch) the
 *                 [Message](https://docs.spotflow.io/send-data/#message) is a part of.
 *                 Use `NULL` if you don't want to specify it.
 * @param message_id (Optional) The ID of the [Message](https://docs.spotflow.io/send-data/#message).
 *                   Use `NULL` if you don't want to specify it.
 * @param buffer The buffer that contains the [Message](https://docs.spotflow.io/send-data/#message).
 * @param length The length of the buffer in bytes.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid or there is an error in
 *              persisting the message.
 */
enum spotflow_result_t spotflow_client_enqueue_message(spotflow_client_t *client,
                                                       const struct spotflow_message_context_t *message_context,
                                                       const char *batch_id,
                                                       const char *message_id,
                                                       const uint8_t *buffer,
                                                       size_t length);

/**
 * Enqueue a [Message](https://docs.spotflow.io/send-data/#message) to
 * be sent to the Platform.
 *
 * If the [Stream](https://docs.spotflow.io/send-data/#stream) doesn't have a
 * [Message ID Autofill Pattern](https://docs.spotflow.io/send-data/#message-id-autofill-pattern),
 * you must provide `message_id`.
 * If the [Stream](https://docs.spotflow.io/send-data/#stream) groups
 * [Messages](https://docs.spotflow.io/send-data/#message) into
 * [Batches](https://docs.spotflow.io/send-data/#batch) and doesn't have a
 * [Batch ID Autofill Pattern](https://docs.spotflow.io/send-data/#batch-id-autofill-pattern),
 * you must provide `batch_id`. See [User Guide](https://docs.spotflow.io/send-data/) for
 * more details.
 * Optionally, you can provide also `batch_slice_id` to use Batch Slices and `chunk_id` to use Message Chunking.
 *
 * The method returns right after it saves the [Message](https://docs.spotflow.io/send-data/#message) to
 * the queue in the local database file. A background thread asynchronously sends the messages from the queue to the Platform.
 * You can check the number of pending messages in the queue using @ref spotflow_client_get_pending_messages_count.
 *
 * @param client The @ref spotflow_client_t object.
 * @param message_context The options that specify how to send the [Message](https://docs.spotflow.io/send-data/#message).
 * @param batch_id (Optional) The ID of the [Batch](https://docs.spotflow.io/send-data/#batch) the
 *                 [Message](https://docs.spotflow.io/send-data/#message) is a part of.
 *                 Use `NULL` if you don't want to specify it.
 * @param batch_slice_id (Optional) The ID of the Batch Slice the [Message](https://docs.spotflow.io/send-data/#message) is
 *                       a part of. Use `NULL` if you dont' want to specify it.
 * @param message_id (Optional) The ID of the [Message](https://docs.spotflow.io/send-data/#message).
 *                   Use `NULL` if you don't want to specify it.
 * @param chunk_id (Optional) The ID of the Chunk of the [Message](https://docs.spotflow.io/send-data/#message).
 *                 Use `NULL` if you don't want to specify it.
 * @param buffer The buffer that contains the [Message](https://docs.spotflow.io/send-data/#message).
 * @param length The length of the buffer in bytes.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid or there is an error in
 *              persisting the message.
 */
enum spotflow_result_t spotflow_client_enqueue_message_advanced(spotflow_client_t *client,
                                                                const struct spotflow_message_context_t *message_context,
                                                                const char *batch_id,
                                                                const char *batch_slice_id,
                                                                const char *message_id,
                                                                const char *chunk_id,
                                                                const uint8_t *buffer,
                                                                size_t length);

/**
 * Enqueue the manual completion of the current [Batch](https://docs.spotflow.io/send-data/#batch) to
 * be sent to the Platform.
 *
 * The Platform also completes the previous [Batch](https://docs.spotflow.io/send-data/#batch) automatically
 * when the new one starts. Therefore, you might not need to call this method at all.
 * See [User Guide](https://docs.spotflow.io/send-data/) for more details.
 *
 * The method returns right after it saves the batch-completing [Message](https://docs.spotflow.io/send-data/#message) to
 * the queue in the local database file. A background thread asynchronously sends the messages from the queue to the Platform.
 * You can check the number of pending messages in the queue using @ref spotflow_client_get_pending_messages_count.
 *
 * @param client The @ref spotflow_client_t object.
 * @param message_context The options that specify how to send the batch-completing [Message](https://docs.spotflow.io/send-data/#message).
 * @param batch_id The ID of the [Batch](https://docs.spotflow.io/send-data/#batch) that will be completed.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid or there is an error in
 *              persisting the message.
 */
enum spotflow_result_t spotflow_client_enqueue_batch_completion(spotflow_client_t *client,
                                                                const struct spotflow_message_context_t *message_context,
                                                                const char *batch_id);

/**
 * Enqueue the manual completion of the current [Message](https://docs.spotflow.io/send-data/#message) to
 * be sent to the Platform. Use this methods when Message Chunking is used.
 *
 * The method returns right after it saves the message-completing Message to the queue in the local database file.
 * A background thread asynchronously sends the Messages from the queue to the Platform.
 * You can check the number of pending messages in the queue using @ref spotflow_client_get_pending_messages_count.
 *
 * @param client The @ref spotflow_client_t object.
 * @param message_context The options that specify how to send the message-completing [Message](https://docs.spotflow.io/send-data/#message).
 * @param batch_id The ID of the [Batch](https://docs.spotflow.io/send-data/#batch) that
 *                 the [Message](https://docs.spotflow.io/send-data/#message) belongs to.
 * @param message_id The ID of the [Message](https://docs.spotflow.io/send-data/#message).
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid or there is an error in
 *              persisting the message.
 */
enum spotflow_result_t spotflow_client_enqueue_message_completion(spotflow_client_t *client,
                                                                  const struct spotflow_message_context_t *message_context,
                                                                  const char *batch_id,
                                                                  const char *message_id);

/**
 * Block the current thread until all the [Messages](https://docs.spotflow.io/send-data/#message) that
 * have been previously enqueued are sent to the Platform.
 *
 * @param client The @ref spotflow_client_t object.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if the argument is invalid.
 */
enum spotflow_result_t spotflow_client_wait_enqueued_messages_sent(spotflow_client_t *client);

/**
 * Send a [Message](https://docs.spotflow.io/send-data/#message) to
 * the Platform.
 *
 * **Warning:** This method blocks the current thread until the Message (and all the Messages enqueued before it)
 * is sent to the Platform. If your Device doesn't have a stable Internet connection, consider using `spotflow_client_enqueue_message` instead.
 *
 * If the [Stream](https://docs.spotflow.io/send-data/#stream) doesn't have a
 * [Message ID Autofill Pattern](https://docs.spotflow.io/send-data/#message-id-autofill-pattern),
 * you must provide `message_id`.
 * If the [Stream](https://docs.spotflow.io/send-data/#stream) groups
 * [Messages](https://docs.spotflow.io/send-data/#message) into
 * [Batches](https://docs.spotflow.io/send-data/#batch) and doesn't have a
 * [Batch ID Autofill Pattern](https://docs.spotflow.io/send-data/#batch-id-autofill-pattern),
 * you must provide `batch_id`. See [User Guide](https://docs.spotflow.io/send-data/) for
 * more details.
 *
 * @param client The @ref spotflow_client_t object.
 * @param message_context The options that specify how to send the [Message](https://docs.spotflow.io/send-data/#message).
 * @param batch_id (Optional) The ID of the [Batch](https://docs.spotflow.io/send-data/#batch) the
 *                 [Message](https://docs.spotflow.io/send-data/#message) is a part of.
 *                 Use `NULL` if you don't want to specify it.
 * @param message_id (Optional) The ID of the [Message](https://docs.spotflow.io/send-data/#message).
 *                   Use `NULL` if you don't want to specify it.
 * @param buffer The buffer that contains the [Message](https://docs.spotflow.io/send-data/#message).
 * @param length The length of the buffer in bytes.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid or there is an error in
 *              persisting the message.
 */
enum spotflow_result_t spotflow_client_send_message(spotflow_client_t *client,
                                                    const struct spotflow_message_context_t *message_context,
                                                    const char *batch_id,
                                                    const char *message_id,
                                                    const uint8_t *buffer,
                                                    size_t length);

/**
 * Send a [Message](https://docs.spotflow.io/send-data/#message) to
 * the Platform.
 *
 * **Warning:** This method blocks the current thread until the Message (and all the Messages enqueued before it)
 * is sent to the Platform. If your Device doesn't have a stable Internet connection, consider using `spotflow_client_enqueue_message_advanced` instead.
 *
 * If the [Stream](https://docs.spotflow.io/send-data/#stream) doesn't have a
 * [Message ID Autofill Pattern](https://docs.spotflow.io/send-data/#message-id-autofill-pattern),
 * you must provide `message_id`.
 * If the [Stream](https://docs.spotflow.io/send-data/#stream) groups
 * [Messages](https://docs.spotflow.io/send-data/#message) into
 * [Batches](https://docs.spotflow.io/send-data/#batch) and doesn't have a
 * [Batch ID Autofill Pattern](https://docs.spotflow.io/send-data/#batch-id-autofill-pattern),
 * you must provide `batch_id`. See [User Guide](https://docs.spotflow.io/send-data/) for
 * more details.
 * Optionally, you can provide also `batch_slice_id` to use Batch Slices and `chunk_id` to use Message Chunking.
 *
 * @param client The @ref spotflow_client_t object.
 * @param message_context The options that specify how to send the [Message](https://docs.spotflow.io/send-data/#message).
 * @param batch_id (Optional) The ID of the [Batch](https://docs.spotflow.io/send-data/#batch) the
 *                 [Message](https://docs.spotflow.io/send-data/#message) is a part of.
 *                 Use `NULL` if you don't want to specify it.
 * @param batch_slice_id (Optional) The ID of the Batch Slice the [Message](https://docs.spotflow.io/send-data/#message) is
 *                       a part of. Use `NULL` if you dont' want to specify it.
 * @param message_id (Optional) The ID of the [Message](https://docs.spotflow.io/send-data/#message).
 *                   Use `NULL` if you don't want to specify it.
 * @param chunk_id (Optional) The ID of the Chunk of the [Message](https://docs.spotflow.io/send-data/#message).
 *                 Use `NULL` if you don't want to specify it.
 * @param buffer The buffer that contains the [Message](https://docs.spotflow.io/send-data/#message).
 * @param length The length of the buffer in bytes.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid or there is an error in
 *              persisting the message.
 */
enum spotflow_result_t spotflow_client_send_message_advanced(spotflow_client_t *client,
                                                             const struct spotflow_message_context_t *message_context,
                                                             const char *batch_id,
                                                             const char *batch_slice_id,
                                                             const char *message_id,
                                                             const char *chunk_id,
                                                             const uint8_t *buffer,
                                                             size_t length);

/**
 * Get the number of [Messages](https://docs.spotflow.io/send-data/#message) that
 * have been persisted in the local database file but haven't been sent to the Platform yet.
 *
 * @param client The @ref spotflow_client_t object.
 * @param count (Output) The number of pending [Messages](https://docs.spotflow.io/send-data/#message).
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_ERROR if any argument is invalid or there is an error in
 *              accessing the local database file.
 */
enum spotflow_result_t spotflow_client_get_pending_messages_count(const spotflow_client_t *client,
                                                                  size_t *count);

/**
 * Write the ID of the [Workspace](https://docs.spotflow.io/manage-access/workspaces/) to which the
 * [Device](https://docs.spotflow.io/connect-devices/#device) belongs into the provided buffer.
 *
 * @param client The @ref spotflow_client_t object.
 * @param buffer The buffer where the [Workspace ID](https://docs.spotflow.io/manage-access/workspaces/) string
 *               including the trailing NUL character will be written to.
 * @param buffer_length The length of the buffer in bytes. Use @ref SPOTFLOW_WORKSPACE_ID_MAX_LENGTH to be sure that it is
 *                      always large enough.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_INSUFFICIENT_BUFFER if the buffer is too small,
 *         @ref SPOTFLOW_ERROR if any argument is invalid.
 */
enum spotflow_result_t spotflow_client_get_workspace_id(const spotflow_client_t *client,
                                                        char *buffer,
                                                        size_t buffer_length);

/**
 * Write the [Device ID](https://docs.spotflow.io/connect-devices/#device-id) into
 * the provided buffer. Note that the value might differ from the one requested in @ref
 * spotflow_client_options_set_device_id if the technician overrides it during the approval of the
 * [Provisioning Operation](https://docs.spotflow.io/connect-devices/#provisioning-operation).
 *
 * @param client The @ref spotflow_client_t object.
 * @param buffer The buffer where the [Device ID](https://docs.spotflow.io/connect-devices/#device-id) string
 *               including the trailing NUL character will be written to.
 * @param buffer_length The length of the buffer in bytes. Use @ref SPOTFLOW_DEVICE_ID_MAX_LENGTH to be sure that it is
 *                      always large enough.
 * @return @ref SPOTFLOW_OK if the function succeeds, @ref SPOTFLOW_INSUFFICIENT_BUFFER if the buffer is too small,
 *         @ref SPOTFLOW_ERROR if any argument is invalid.
 */
enum spotflow_result_t spotflow_client_get_device_id(const spotflow_client_t *client,
                                                     char *buffer,
                                                     size_t buffer_length);

/**
 * Disconnect from the Platform and destroy the @ref spotflow_client_t object.
 *
 * @param client The @ref spotflow_client_t object.
 */
void spotflow_client_destroy(spotflow_client_t *client);

#endif /* spotflow_SDK_H */
